# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.test import TestCase
from django.test import RequestFactory
from mfw.tests.override_settings import override_settings
from mfw.device.base import Device
from mfw.middleware.flavour import DeviceFlavourDetectionMiddleware
from mfw.middleware.flavour import parse_device
from mfw.middleware.flavour import get_flavour


class MFWDeviceFlavourDetectionMiddlewareTestCase(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_standalone(self):
        request = self.factory.get('/')
        request.device = Device(True, 'smartphone', 'iOS', 'iPhone', '5.0')
        middleware = DeviceFlavourDetectionMiddleware()

        middleware.process_request(request)

        self.assertTrue(hasattr(request, 'flavour'))
        self.assertTrue(isinstance(request.flavour, basestring))

        flavour = get_flavour()
        self.assertEqual(request.flavour, flavour)

    @override_settings(
            MFW_DEVICE_FLAVOUR_COLUMNS=lambda device: (device.kind, device.name, device.model, device.version),
            MFW_DEVICE_FLAVOUR_OVERLAP_RULES=(),
        )
    def test_parse_device(self):
        device = Device(
                support_cookie=True,
                kind='smartphone',
                name='iOS',
                model='iPhone',
                version='5.0'
            )
        self.assertEqual(parse_device(device), 'smartphone/iOS/iPhone/5.0')

        device = Device(
                support_cookie=True,
                kind='smartphone',
                name='iOS',
                model='iPhone',
                version=None
            )
        self.assertEqual(parse_device(device), 'smartphone/iOS/iPhone')

        device = Device(
                support_cookie=True,
                kind='smartphone',
                name=None,
                model='iPhone',
                version=None
            )
        self.assertEqual(parse_device(device), 'smartphone/iPhone')

    @override_settings(
            MFW_DEVICE_FLAVOUR_COLUMNS=lambda device: (device.kind, device.name, device.model, device.version),
            MFW_DEVICE_FLAVOUR_OVERLAP_RULES=(
                # Add 'fossil' to pattern name if no cookie is supported by the device
                (lambda device, previous: not device.support_cookie, lambda device, previous: ['fossil']+previous),
                # Use 'webkit' for Safari and Chrome and do not use `model` even model
                # is not None
                (lambda device, previous: device.name in ('Chrome', 'Safari'), lambda device, previous: (device.kind, 'webkit', device.version)),
            ))
    def test_parse_device_with_overlap_rules(self):
        device = Device(
                support_cookie=False,
                kind='mobilephone',
                name='DoCoMo',
                model='D502i',
                version=None,
            )
        self.assertEqual(parse_device(device), 'fossil/mobilephone/DoCoMo/D502i')

        device = Device(
                support_cookie=True,
                kind='browser',
                name='Safari',
                model='Windows',
                version='5.0',
            )
        self.assertEqual(parse_device(device), 'browser/webkit/5.0')

        device = Device(
                support_cookie=True,
                kind='browser',
                name='Chrome',
                model='Windows',
                version='5.0',
            )
        self.assertEqual(parse_device(device), 'browser/webkit/5.0')

        device = Device(
                support_cookie=True,
                kind='browser',
                name='Firefox',
                model='Windows',
                version='5.0',
            )
        self.assertEqual(parse_device(device), 'browser/Firefox/Windows/5.0')




# vim: set fileencoding=utf-8 :
"""
Device base


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import uuid
from django.conf import settings

class Device(object):
    _support_cookie = None
    _kind = None
    _name = None
    _model = None
    _version = None
    _encoding = None
    is_dummy = False

    def __init__(self, support_cookie=None, kind=None, name=None, model=None, version=None, encoding=None):
        self._support_cookie = self._support_cookie or support_cookie
        self._kind = self._kind or kind
        self._name = self._name or name
        self._model = self._model or model
        self._version = self._version or version
        self._encoding = self._encoding or encoding

    @property
    def support_cookie(self):
        return self._support_cookie or False

    @property
    def kind(self):
        return self._kind or ''

    @property
    def name(self):
        return self._name or ''

    @property
    def model(self):
        return self._model or ''

    @property
    def version(self):
        return self._version or ''

    @property
    def encoding(self):
        return self._encoding or settings.DEFAULT_CHARSET

    def __hash__(self):
        guid = uuid.uuid5(uuid.NAMESPACE_DNS, "%s-%s-%s-%s-%s-%s" % (
                self.support_cookie,
                self.kind,
                self.name,
                self.model,
                self.version,
                self.encoding,
            ))
        return hash(str(guid))

    @classmethod
    def detect(cls, meta):
        raise NotImplementedError


class UserAgentRegexPatternDevice(Device):
    _patterns = None

    @classmethod
    def detect(cls, meta):
        user_agent = meta.get('HTTP_USER_AGENT', None)
        if not user_agent:
            return None

        cache_name = "_%s_cache" % user_agent
        if hasattr(cls, cache_name):
            return getattr(cls, cache_name)

        for pattern in cls._patterns:
            name, pattern = pattern
            m = pattern.match(user_agent)
            if m:
                device = cls(name=name, **m.groupdict())
                setattr(cls, cache_name, device)
                return device
        return None

class DummyDevice(Device):
    is_dummy = True
    _support_cookie = True




# vim: set fileencoding=utf-8 :
"""
Mini blog models

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
__VERSION__ = "0.1.0"
from django.db import models
from django.utils.text import ugettext_lazy as _

class EntryBase(models.Model):
    """A base abstract model of Entry"""
    title = models.CharField(_('title'), max_length=50, unique=True)
    body = models.TextField(_('body'))

    created_at = models.DateTimeField(_('date and time created'),
            auto_now_add=True)
    updated_at = models.DateTimeField(_('date and time updated'),
            auto_now=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('blogs-entry-detail', (), {'slug': self.title})

    def clean(self):
        """custom validation"""
        from django.core.exceptions import ValidationError
        if self.title in ('create', 'update', 'delete'):
            raise ValidationError(
                    """The title cannot be 'create', 'update' or 'delete'""")

class Entry(EntryBase):
    """mini blog entry model
    
    >>> entry = Entry()

    # Attribute test
    >>> assert hasattr(entry, 'title')
    >>> assert hasattr(entry, 'body')
    >>> assert hasattr(entry, 'created_at')
    >>> assert hasattr(entry, 'updated_at')

    # Function test
    >>> assert callable(getattr(entry, '__unicode__'))
    >>> assert callable(getattr(entry, 'get_absolute_url'))
    """
    pass




# vim: set fileencoding=utf-8 :
"""
Japanese Emoji translation middleware


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from django.conf import settings
from django.utils.encoding import force_unicode

import e4u

def translate(src, carrier, encoding):
    c = force_unicode(src)
    # translate all emoji
    c = e4u.translate(c, reverse=True, **{'carrier': carrier, 'encoding': encoding})
    return c

def encode(src, encoding):
    c = force_unicode(src)
    # insted of raise exception, replace character which cannot encode.
    dst = c.encode(encoding, 'replace')
    return dst

def translate_dict(src, carrier):
    src._mutable = True
    for key, value in src.iteritems():
        src[key] = e4u.translate(value, **{'carrier': carrier, 'encoding': settings.DEFAULT_CHARSET})
    src._mutable = False
    src._mfw_encoded = True


class DeviceEmojiTranslationMiddleware(object):
    """Translate device emoji to unicode emoji. DO NOT USE this with `mfw.middleware.encoding.DeviceEncodingMiddleware`."""
    def process_request(self, request):
        # tell django to use device encoding
        if not hasattr(request.device, 'carrier'):
            return

        # translate GET/POST
        if not getattr(request.GET, '_mfw_translated', False):
            translate_dict(request.GET, request.device.carrier)
        if not getattr(request.POST, '_mfw_translated', False):
            translate_dict(request.POST, request.device.carrier)
    
    def process_response(self, request, response):
        if response['content-type'].startswith('text/'):
            if not getattr(response, '_mfw_translated', False):
                if not hasattr(request.device, 'carrier'):
                    carrier = settings.MFW_EMOJI_DEFAULT_CARRIER
                else:
                    carrier = request.device.carrier
                encoding = request.device.encoding
                response.content = translate(response.content, carrier, encoding)
                response._mfw_translated = True

        return response



# vim: set fileencoding=utf-8 :
"""
Sample HTTP_USER_AGENT


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement

INTERNET_EXPLORER = (
        # Kind, Name, Model, Version, USER_AGENT
        ('browser', 'Explorer', 'Windows', '4.0', 'Mozilla/4.0 (compatible; MSIE 4.01; Windows NT)'),
        ('browser', 'Explorer', 'Mac', '4.5', 'Mozilla/4.0 (compatible; MSIE 4.5; Mac_PowerPC)'),
        ('browser', 'Explorer', 'Windows', '5.0', 'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; .NET CLR 1.1.4322)'),
        ('browser', 'Explorer', 'Mac', '5.2', 'Mozilla/4.0 (compatible; MSIE 5.23; Mac_PowerPC)'),
        ('browser', 'Explorer', 'Windows', '5.5', 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0; by TSG)'),
        ('browser', 'Explorer', 'Windows', '6.0', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows XP)'),
        ('browser', 'Explorer', 'Windows', '7.0', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)'),
        ('browser', 'Explorer', 'Windows', '8.0', 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1;'),
        ('browser', 'Explorer', 'Windows', '9.0', 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)'),
        ('browser', 'Explorer', 'Windows', '10.0', 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)'),
    )

GOOGLE_CHROME = (
        ('browser', 'Chrome', 'Windows', '1.0', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/525.19 (KHTML, like Gecko) Chrome/1.0.154.48 Safari/525.19',),
        ('browser', 'Chrome', 'Windows', '2.0', 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/530.5 (KHTML, like Gecko) Chrome/2.0.172.33 Safari/530.5',),
        ('browser', 'Chrome', 'Windows', '3.0', 'Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/532.0 (KHTML, like Gecko) Chrome/3.0.195.38 Safari/532.0',),
        ('browser', 'Chrome', 'Mac', '11.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.71 Safari/534.24',),
        ('browser', 'Chrome', 'Mac', '11.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.24 (KHTML, like Gecko) Iron/11.0.700.2 Chrome/11.0.700.2 Safari/534.24',),
        ('browser', 'Chrome', 'Windows', '11.0', 'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/534.24 (KHTML, like Gecko) Chrome/11.0.696.65 Safari/534.24',),
        ('browser', 'Chrome', 'Mac', '12.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.112 Safari/534.30',),
        ('browser', 'Chrome', 'Windows', '13.0', 'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.107 Safari/535.1',),
        ('browser', 'Chrome', 'Mac', '13.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) RockMelt/0.9.64.361 Chrome/13.0.782.218 Safari/535.1',),
        ('browser', 'Chrome', 'Mac', '13.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.112 Safari/535.1',),
        ('browser', 'Chrome', 'Mac', '13.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.220 Safari/535.1',),
        ('browser', 'Chrome', 'Mac', '14.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1',),
        ('browser', 'Chrome', 'Windows', '14.0', 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.202 Safari/535.1',),
        ('browser', 'Chrome', 'Windows', '15.0', 'Mozilla/5.0 (Windows NT 5.1) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.120 Safari/535.2',),
        ('browser', 'Chrome', 'Linux', '15.0', 'Mozilla/5.0 (X11; Linux i686) AppleWebKit/535.2 (KHTML, like Gecko) Ubuntu/10.04 Chromium/15.0.874.106 Chrome/15.0.874.106 Safari/535.2',),
        ('browser', 'Chrome', 'Mac', '15.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_2) AppleWebKit/535.2 (KHTML, like Gecko) Chrome/15.0.874.106 Safari/535.2',),
        ('browser', 'Chrome', 'Windows', '16.0', 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7',),
        ('browser', 'Chrome', 'Mac', '16.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7',),
    )

LUNASCAPE = (
        ('browser', 'Lunascape', 'Windows', '4.7', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; SLCC1; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30618; Lunascape 4.7.3)',),
        ('browser', 'Lunascape', 'Windows', '5.0', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; Lunascape 5.0 alpha2)',),
        ('browser', 'Lunascape', 'Windows', '5.0', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.1; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729; Lunascape 5.0 alpha2)',),
        ('browser', 'Lunascape', 'Windows', '5.0', 'Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1) ; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506; Tablet PC 2.0; Lunascape 5.0 alpha2)',),
        ('browser', 'Lunascape', 'Windows', '6.5', 'Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; MAGW; .NET4.0C; Lunascape 6.5.8.24780)',),
    )

FIREFOX = (
        ('browser', 'Firefox', 'Windows', '6.0', 'Mozilla/5.0 (Windows NT 5.1; rv:6.0) Gecko/20100101 Firefox/6.0',),
        ('browser', 'Firefox', 'Mac', '6.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:6.0.2) Gecko/20100101 Firefox/6.0.2',),
        ('browser', 'Firefox', 'Windows', '7.0', 'Mozilla/5.0 (Windows NT 6.0; rv:7.0.1) Gecko/20100101 Firefox/7.0.1',),
        ('browser', 'Firefox', 'Mac', '7.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:7.0.1) Gecko/20100101 Firefox/7.0.1',),
        ('browser', 'Firefox', 'Mac', '8.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.5; rv:8.0.1) Gecko/20100101 Firefox/8.0.1',),
        ('browser', 'Firefox', 'Windows', '9.0', 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:9.0.1) Gecko/20100101 Firefox/9.0.1',),
        ('browser', 'Firefox', 'Mac', '9.0', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:9.0.1) Gecko/20100101 Firefox/9.0.1',),
    )

SAFARI = (
        ('browser', 'Safari', 'Mac', '3.1', 'Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; ja-jp) AppleWebKit/525.13 (KHTML, like Gecko) Version/3.1 Safari/525.13',),
        ('browser', 'Safari', 'Mac', '3.1', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_3; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.1 Safari/525.20',),
        ('browser', 'Safari', 'Mac', '3.1', 'Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_4_11; ja-jp) AppleWebKit/525.18 (KHTML, like Gecko) Version/3.1.2 Safari/525.22',),
        ('browser', 'Safari', 'Mac', '3.2', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_5; ja-jp) AppleWebKit/525.26.2 (KHTML, like Gecko) Version/3.2 Safari/525.26.12',),
        ('browser', 'Safari', 'Mac', '3.2', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/525.27.1 (KHTML, like Gecko) Version/3.2.1 Safari/525.27.1',),
        ('browser', 'Safari', 'Mac', '4.0', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_6; ja-jp) AppleWebKit/528.16 (KHTML, like Gecko) Version/4.0 Safari/528.16',),
        ('browser', 'Safari', 'Mac', '4.0', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; ja-jp) AppleWebKit/531.21.8 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10',),
        ('browser', 'Safari', 'Mac', '4.0', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/531.21.11 (KHTML, like Gecko) Version/4.0.4 Safari/531.21.10',),
        ('browser', 'Safari', 'Mac', '5.0', 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_3; ja-jp) AppleWebKit/533.16 (KHTML, like Gecko) Version/5.0 Safari/533.16',),
        ('browser', 'Safari', 'Mac', '5.1', 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/534.52.7 (KHTML, like Gecko) Version/5.1.2 Safari/534.52.7',),
    )

OPERA = (
        ('browser', 'Opera', 'Windows', '9.0', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; ja) Opera 9.00',),
        ('browser', 'Opera', 'Windows', '9.0', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; en) Opera 9.00',),
        ('browser', 'Opera', 'Windows', '9.0', 'Opera/9.00 (Windows NT 5.1; U; ja)',),
        ('browser', 'Opera', 'Windows', '9.6', 'Opera/9.60 (Windows NT 5.1; U; ja) Presto/2.1.1',),
        ('browser', 'Opera', 'Windows', '9.6', 'Opera/9.61 (Windows NT 5.1; U; ja) Presto/2.1.1',),
        ('browser', 'Opera', 'Windows', '9.6', 'Opera/9.62 (Windows NT 5.1; U; ja) Presto/2.1.1',),
    )

IPHONE = (
        ('smartphone', 'iOS', 'iPhone', '', 'Mozilla/5.0 (iPhone; U; CPU like Mac OS X; en) AppleWebKit/420+ (KHTML, like Gecko) Version/3.0 Mobile/1C28 Safari/419.3',),
        ('smartphone', 'iOS', 'iPhone', '2.1', 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 2_1 like Mac OS X; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F136 Safari/525.20',),
        ('smartphone', 'iOS', 'iPhone', '3.1', 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_1_3 like Mac OS X; ja-jp) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7E18 Safari/528.16',),
        ('smartphone', 'iOS', 'iPhone', '4.3', 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_5 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8L1',),
        ('smartphone', 'iOS', 'iPhone', '5.0', 'Mozilla/5.0 (iPhone; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Mobile/9A405 Safari/7534.48.3',),
    )

IPOD = (
        ('smartphone', 'iOS', 'iPod', '', 'Mozilla/5.0 (iPod; U; CPU like Mac OS X; en) AppleWebKit/420.1 (KHTML, like Gecko) Version/3.0 Mobile/3A100a Safari/419.3',),
        ('smartphone', 'iOS', 'iPod', '2.1', 'Mozilla/5.0 (iPod; U; CPU iPhone OS 2_1 like Mac OS X; ja-jp) AppleWebKit/525.18.1 (KHTML, like Gecko) Version/3.1.1 Mobile/5F137 Safari/525.20',),
        ('smartphone', 'iOS', 'iPod', '4.1', 'Mozilla/5.0 (iPod; U; CPU iPhone OS 4_1 like Mac OS X; ja-jp) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B118 Safari/6531.22.7',),
        ('smartphone', 'iOS', 'iPod', '4.2', 'Mozilla/5.0 (iPod; U; CPU iPhone OS 4_2_1 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPod', '4.3', 'Mozilla/5.0 (iPod; U; CPU iPhone OS 4_3_5 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPod', '5.0', 'Mozilla/5.0 (iPod; CPU iPhone OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3',),
    )

IPAD = (
        ('smartphone', 'iOS', 'iPad', '3.2', 'Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B367 Safari/531.21.10',),
        ('smartphone', 'iOS', 'iPad', '3.2', 'Mozilla/5.0 (iPad; U; CPU OS 3_2_1 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Mobile/7B405',),
        ('smartphone', 'iOS', 'iPad', '4.2', 'Mozilla/5.0 (iPad; U; CPU OS 4_2 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Mobile/8C134',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8F190 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3_1 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8G4 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3_2 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3_3 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8J2 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3_4 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8K2 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '4.3', 'Mozilla/5.0 (iPad; U; CPU OS 4_3_5 like Mac OS X; ja-jp) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8L1 Safari/6533.18.5',),
        ('smartphone', 'iOS', 'iPad', '5.0', 'Mozilla/5.0 (iPad; CPU OS 5_0_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A405 Safari/7534.48.3',),
    )

ANDROID = (
        ('smartphone', 'Android', 'GDDJ-09', '1.5', 'Mozilla/5.0 (Linux; U; Android 1.5; ja-jp; GDDJ-09 Build/CDB56) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'IS01', '1.6', 'Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/S3082) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'IS01', '1.6', 'Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; IS01 Build/SA180) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'Docomo HT-03A', '1.6', 'Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; Docomo HT-03A Build/DRD08) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'SonyEricssonSO-01B', '1.6', 'Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; SonyEricssonSO-01B Build/R1EA029) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'generic', '1.6', 'Mozilla/5.0 (Linux; U; Android 1.6; ja-jp; generic Build/Donut) AppleWebKit/528.5+ (KHTML, like Gecko) Version/3.1.2 Mobile Safari/525.20.1',),
        ('smartphone', 'Android', 'SonyEricssonSO-01B', '2.1', 'Mozilla/5.0 (Linux; U; Android 2.1-update1; ja-jp; SonyEricssonSO-01B Build/2.0.2.B.0.29) AppleWebKit/530.17 (KHTML, like Gecko) Version/4.0 Mobile Safari/530.17',),
        ('smartphone', 'Android', 'Full Android', '2.2', 'Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; Full Android Build/MASTER) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'IS03', '2.2', 'Mozilla/5.0 (Linux; U; Android 2.2.1; ja-jp; IS03 Build/S9090) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'SC-02C', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SC-02C Build/GINGERBREAD) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'INFOBAR A01', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; INFOBAR A01 Build/S9081) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', '001HT', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; 001HT Build/GRI40) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'SonyEricssonX10i', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.3; ja-jp; SonyEricssonX10i Build/3.0.1.G.0.75) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'SonyEricssonIS11S', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; SonyEricssonIS11S Build/4.0.1.B.0.112) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'IS05', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.4; ja-jp; IS05 Build/S9290) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'F-05D', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; F-05D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'T-01D', '2.3', 'Mozilla/5.0 (Linux; U; Android 2.3.5; ja-jp; T-01D Build/F0001) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1',),
        ('smartphone', 'Android', 'MZ604', '3.0', 'Mozilla/5.0 (Linux; U; Android 3.0.1; ja-jp; MZ604 Build/H.6.2-20) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'K1', '3.1', 'Mozilla/5.0 (Linux; U; Android 3.1; en-us; K1 Build/HMJ37) AppleWebKit/534.13(KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'AT100', '3.1', 'Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; AT100 Build/HMJ37) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'Sony Tablet S', '3.1', 'Mozilla/5.0 (Linux; U; Android 3.1; ja-jp; Sony Tablet S Build/THMAS10000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'SC-01D', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; SC-01D Build/MASTER) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'AT1S0', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; AT1S0 Build/HTJ85B) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'F-01D', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; F-01D Build/F0001) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'Sony Tablet S', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; Sony Tablet S Build/THMAS11000) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'A01SH', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2; ja-jp; A01SH Build/HTJ85B) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Safari/533.1',),
        ('smartphone', 'Android', 'Transformer TF101', '3.2', 'Mozilla/5.0 (Linux; U; Android 3.2.1; ja-jp; Transformer TF101 Build/HTK75) AppleWebKit/534.13 (KHTML, like Gecko) Version/4.0 Safari/534.13',),
        ('smartphone', 'Android', 'Galaxy Nexus', '4.0', 'Mozilla/5.0 (Linux; U; Android 4.0.1; ja-jp; Galaxy Nexus Build/ITL41D) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30',),
        ('smartphone', 'Android', 'Opera Mobi', '2.3', 'Opera/9.80 (Android 2.3.3; Linux; Opera Mobi/ADR-1111101157; U; ja) Presto/2.9.201 Version/11.50',),
        ('smartphone', 'Android', 'Opera Tablet', '3.2', 'Opera/9.80 (Android 3.2.1; Linux; Opera Tablet/ADR-1109081720; U; ja) Presto/2.8.149 Version/11.10',),
        ('smartphone', 'Android', 'Firefox', '9.0', 'Mozilla/5.0 (Android; Linux armv7l; rv:9.0) Gecko/20111216 Firefox/9.0 Fennec/9.0',),
    )

WINDOWS_PHONE = (
        ('smartphone', 'WindowsPhone', '', '6.5', 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; KDDI-TS01; Windows Phone 6.5.3.5)',),
        ('smartphone', 'WindowsPhone', '', '7.5', 'Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; FujitsuToshibaMobileCommun; IS12T; KDDI)',),
    )

BLACK_BERRY = (
        ('smartphone', 'BlackBerry', '', '4.6', 'BlackBerry9000/4.6.0.294 Profile/MIDP-2.0 Configuration/CLDC-1.1 VendorID/220',),
        ('smartphone', 'BlackBerry', '', '5.0', 'BlackBerry9300/5.0.0.1007 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/220',),
        ('smartphone', 'BlackBerry', '', '5.0', 'BlackBerry9700/5.0.0.1014 Profile/MIDP-2.1 Configuration/CLDC-1.1 VendorID/220',),
        ('smartphone', 'BlackBerry', '', '6.0', 'Mozilla/5.0 (BlackBerry; U; BlackBerry 9700; ja) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.570 Mobile Safari/534.8+',),
        ('smartphone', 'BlackBerry', '', '6.0', 'Mozilla/5.0 (BlackBerry; U; BlackBerry 9780; ja) AppleWebKit/534.8+ (KHTML, like Gecko) Version/6.0.0.587 Mobile Safari/534.8+',),
        ('smartphone', 'BlackBerry', '', '7.1', 'Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; ja) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.74 Mobile Safari/534.11+',),
        ('smartphone', 'BlackBerry', 'Opera Mini', '9.8', 'Opera/9.80 (BlackBerry; Opera Mini/6.1.25376/26.958; U; en) Presto/2.8.119 Version/10.54',),
    )

DOCOMO_IP = '210.153.84.0'
DOCOMO = (
        # Kind, Name, Model, Version, Cookie, HTTP_USER_AGENT, REMOTE_ADDR, Additional
        ('mobilephone', 'DoCoMo', 'D502i', '', False, 'DoCoMo/1.0/D502i', DOCOMO_IP, {'HTTP_X_DCMGUID': 'A12345'}),
        ('mobilephone', 'DoCoMo', 'F06B', '', True, 'DoCoMo/2.0 F06B(c500;TB;W24H16)', DOCOMO_IP, {'HTTP_X_DCMGUID': 'A12345'}),
        ('mobilephone', 'DoCoMo', 'F06B', '', True, 'DoCoMo/2.0 F06B(c500;TB;W24H16)', '127.0.0.1', {'HTTP_X_DCMGUID': 'A12345'}),
    )

KDDI_IP = '210.230.128.224'
KDDI = (
        ('mobilephone', 'KDDI', 'SA31', '', True, 'KDDI-SA31 UP.Browser/6.2.0.7.3.129 (GUI) MMP/2.0', KDDI_IP, {'HTTP_X_UP_SUBNO': 'A12345'}),
        ('mobilephone', 'KDDI', 'SA31', '', True, 'SIE-SA31 UP.Browser/6.2.0.7.3.129 (GUI) MMP/2.0', KDDI_IP, {'HTTP_X_UP_SUBNO': 'A12345'}),
        ('mobilephone', 'KDDI', 'SA31', '', True, 'KDDI-SA31 UP.Browser/6.2.0.7.3.129 (GUI) MMP/2.0', '127.0.0.1', {'HTTP_X_UP_SUBNO': 'A12345'}),
    )

SOFTBANK_IP = '123.108.237.0'
SOFTBANK = (
        ('mobilephone', 'Softbank', 'J-SH02', '', False, 'J-PHONE/2.0/J-SH02', SOFTBANK_IP, {'HTTP_X_JPHONE_UID': 'A12345'}),
        ('mobilephone', 'Softbank', 'V803T', '', True, 'Vodafone/1.0/V803T/TJ001[/Serial] Browser/VF-Browser/1.0', SOFTBANK_IP, {'HTTP_X_JPHONE_UID': 'A12345'}),
        ('mobilephone', 'Softbank', '002Pe', '', True, 'SoftBank/1.0/002Pe/PJP10[/Serial] Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1', SOFTBANK_IP, {'HTTP_X_JPHONE_UID': 'A12345'}),
        ('mobilephone', 'Softbank', '002Pe', '', True, 'SoftBank/1.0/002Pe/PJP10[/Serial] Browser/NetFront/3.4 Profile/MIDP-2.0 Configuration/CLDC-1.1', '127.0.0.1', {'HTTP_X_JPHONE_UID': 'A12345'}),
    )



# vim: set fileencoding=utf-8 :
"""
csrf related views
~~~~~~~~~~~~~~~~~~

debug_csrf_failure()
~~~~~~~~~~~~~~~~~~~~
Display the normal debug page and not the minimal csrf debug page.
usage: Add this to your settings:
-----------------------------------------------------------------------
CSRF_FAILURE_VIEW='django_tools.views.csrf.debug_csrf_failure'
-----------------------------------------------------------------------


:copyleft: 2012 by the django-tools team, see AUTHORS for more details.
:license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.views.csrf import csrf_failure
from django.conf import settings
from django.http import Http404


class CsrfFailure(Exception):
    pass


def debug_csrf_failure(request, reason=""):
    """
raised own CsrfFailure() exception to get the normal debug page on
Csrf failures.
See also:
https://docs.djangoproject.com/en/1.3/ref/contrib/csrf/#rejected-requests
More Info: See DocString above.
"""
    if not settings.DEBUG:
        # Use original HttpResponseForbidden:
        return csrf_failure(request, reason)

    raise CsrfFailure("csrf failure debug: %r" % reason)



# vim: set fileencoding=utf-8 :
"""
Mobilephone


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import re
from django.conf import settings
from mfw.device.base import Device


def appraisal(device, meta):
    """check carrier CIDR and add ``reliable`` attribute to device instance"""
    if settings.MFW_CHECK_DEVICE_RELIABLE:
        from IPy import IP
        from mfw.cidr import get_cidr
        cidr = get_cidr(device.carrier)
        remote_addr = meta.get('REMOTE_ADDR', None)
        if cidr and remote_addr and IP(remote_addr) in cidr:
            device._reliable = True
    device.__class__.reliable = property(lambda self: getattr(self, '_reliable', False))
    return device


class Mobilephone(Device):
    _kind = 'mobilephone'

    @property
    def carrier(self):
        return self._name.lower()

    @property
    def uid(self):
        return self._uid


class DoCoMo(Mobilephone):
    _name = 'DoCoMo'

    @classmethod
    def detect(cls, meta):
        pattern_mova = re.compile(r"DoCoMo/1.0/(?P<model>[^/]*)(?:/c(?P<cache>\d*)|)")
        pattern_foma = re.compile(r"DoCoMo/2.0 (?P<model>[^\(]*)\(c(?P<cache>\d*)")

        uid = meta.get('HTTP_X_DCMGUID', None)
        user_agent = meta.get('HTTP_USER_AGENT', None)
        if not user_agent:
            return None

        m = pattern_mova.match(user_agent)
        if not m:
            m = pattern_foma.match(user_agent)
        if not m:
            return None
        kwargs = m.groupdict()
        kwargs['cache'] = int(kwargs['cache']) if kwargs.get('cache', None) else 5

        # is this device support cookie?
        cache = kwargs.pop('cache')
        kwargs['support_cookie'] = cache >= 500

        instance = cls(**kwargs)
        instance._encoding = 'cp932'
        instance._uid = uid
        return appraisal(instance, meta)


class KDDI(Mobilephone):
    _name = 'KDDI'

    @classmethod
    def detect(cls, meta):
        pattern = re.compile(r"(?:(?:KDDI-)|(?:SIE-))(?P<model>[^\s/]*)")

        uid = meta.get('HTTP_X_UP_SUBNO', None)
        user_agent = meta.get('HTTP_USER_AGENT', None)
        if not user_agent:
            return None

        m = pattern.match(user_agent)
        if not m:
            return None
        kwargs = m.groupdict()

        kwargs['support_cookie'] = True

        instance = cls(**kwargs)
        instance._encoding = 'cp932'
        instance._uid = uid
        return appraisal(instance, meta)


class Softbank(Mobilephone):
    _name = 'Softbank'

    @classmethod
    def detect(cls, meta):
        pattern = re.compile(r"(?P<generation>J-PHONE|Vodafone|SoftBank)/(?P<version>[\d\.]*)/(?P<model>[^/\[]*)")

        uid = meta.get('HTTP_X_JPHONE_UID', None)
        user_agent = meta.get('HTTP_USER_AGENT', None)
        if not user_agent:
            return None

        m = pattern.match(user_agent)
        if not m:
            return None
        kwargs = m.groupdict()

        # is this device support cookie?
        generation = kwargs.pop('generation')
        version = kwargs.pop('version')
        if generation == 'J-PHONE':
            kwargs['support_cookie'] = version.startswith('5')
        else:
            kwargs['support_cookie'] = True

        instance = cls(**kwargs)
        instance._encoding = 'utf-8'
        instance._uid = uid
        return appraisal(instance, meta)



# vim: set fileencoding=utf-8 :
"""
Flavour middleware to enable flavour template system


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import os
from django.conf import settings
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

_thread_locals = local()
def get_flavour():
    """get stored flavour in threadlocal"""
    return getattr(_thread_locals, 'flavour', None)


def parse_device(device):
    """parse device and return device flavour"""

    # try to load flavour from cache because generating
    # flavour may slow.
    if device in parse_device._cache:
        return parse_device._cache[device]
    
    columns = list(settings.MFW_DEVICE_FLAVOUR_COLUMNS(device))
    for overlap_rule in settings.MFW_DEVICE_FLAVOUR_OVERLAP_RULES:
        condition, overlap = overlap_rule
        if condition(device, columns):
            columns = overlap(device, columns)
            if isinstance(columns, tuple):
                columns = list(columns)

    # remove all empty columns
    columns = filter(bool, columns)

    if columns:
        flavour = os.path.join(*columns)
    else:
        flavour = ''

    parse_device._cache[device] = flavour
    return flavour
parse_device._cache = {}


class DeviceFlavourDetectionMiddleware(object):
    """create ``flavour`` from ``request.device`` and store in ``flavour`` attribute"""
    def process_request(self, request):
        flavour = parse_device(request.device)
        request.flavour = flavour
        # store flavor in thread local
        _thread_locals.flavour = flavour



# vim: set fileencoding=utf-8 :
"""
SessionMiddleware which use UID insted of cookie to store
session key when the device does not support cookie



AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import re
import uuid
from django.conf import settings
from django.shortcuts import redirect
from django.utils.importlib import import_module
from django.contrib.sessions.middleware import SessionMiddleware as OriginalSessionMiddleware


def create_session_key(carrier, uid):
    """create session key with carrier and uid"""
    return str(uuid.uuid5(uuid.NAMESPACE_DNS, carrier+uid))

class SessionMiddleware(OriginalSessionMiddleware):
    """
    SessionMiddleware which use UID insted of cookie to store
    session key when the device does not support cookie

    """
    def process_request(self, request):
        if request.device.support_cookie or not hasattr(request.device, 'uid'):
            # do not use this middleware for device which support cookie or
            # doesn't have uid
            return super(SessionMiddleware, self).process_request(request)

        if not settings.MFW_SESSION_TRUST_NON_RELIABLE_DEVICE and not request.device.reliable:
            # the device cannot be trusted so do not use cache based session
            return super(SessionMiddleware, self).process_request(request)


        if request.device.uid:
            # get session_key from cache
            session_key = create_session_key(request.device.carrier, request.device.uid)
        else:
            # DoCoMo doesn't return uid request in GET without `guid=on` thus redirect
            if request.device.carrier == 'docomo' and request.method == 'GET' and not request.GET.has_key('guid'):
                protocol = 'https' if request.is_secure() else 'http'
                query_string = '&guid=on' if request.GET else '?guid=on'
                url = "%s://%s%s%s" % (
                        protocol,
                        request.get_host(),
                        request.get_full_path(),
                        query_string
                    )
                return redirect(url)
            session_key = None
        
        # create session instance via session_key
        engine = import_module(settings.SESSION_ENGINE)
        request.session = engine.SessionStore(session_key)
        # `_get_new_session_key` rewrite session key thus
        # override the method otherwise session key for
        # particular user will be losted.
        request.session._get_new_session_key = lambda : session_key
        def create(self):
            # _get_new_session_key return same value thus simply
            # over write the existing session data.
            self.session_key = self._get_new_session_key()
            try:
                self.save(must_create=True)
            except CreateError:
                self.save()
            self.modified = True
            self._session_cache = {}
            return
        request.session.__class__.create = create

        # cookie is not available in this device thus override the
        # COOKIES attribute with session instance
        request.COOKIES = request.session
        request.COOKIES._mfw_overridden = True

        return None


    def process_response(self, request, response):
        if getattr(request.COOKIES, '_mfw_overridden', False):
            # cookie is not available thus override set_cookie method
            # of response instance.
            if not getattr(response.set_cookie, '_mfw_overridden', False):
                def set_cookie(key, value, **kwargs):
                    request.session[key] = value
                    request.session.save()
                response.set_cookie = set_cookie
                response.set_cookie._mfw_overridden = True
            if request.device.carrier == 'docomo':
                # Add guid=on to all internal links found in response for
                # handling docomo
                url_pattern = re.compile(r"""((?:action|href)=["'])(?P<url>[^"']*)(["'])""")
                def repl(m):
                    url = m.group('url')
                    if url.startswith('http') or 'guid=on' in url:
                        # non internal link or already has
                        return m.group(0)
                    if '?' not in url:
                        url = url + '?guid=on'
                        return m.expand(r"\1%s\3" % url)
                    else:
                        url = url + '&guid=on'
                        return m.expand(r"\1%s\3" % url)
                response.content = url_pattern.sub(repl, response.content)
        return super(SessionMiddleware, self).process_response(request, response)



from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^registration/', include('django.contrib.auth.urls')),
    url(r'^', include('miniblog.blogs.urls')),
    url(r'^comments/', include('django.contrib.comments.urls')),
)



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
"""
Unittest module of models


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.test import TestCase
from django.contrib.auth.models import User

from ..models import Entry

class EntryModelTestCase(TestCase):
    def setUp(self):
        try:
            self.admin = User.objects.get(pk=1)
        except User.DoesNotExist:
            self.admin = User.objects.create_user(
                    username='admin', email='admin@test.com', password='password')

    def test_creation(self):
        """blogs.Entry: creation works correctly"""
        entry = Entry(title='foo', body='bar')
        entry.full_clean()
        self.assertEqual(entry.title, 'foo')
        self.assertEqual(entry.body, 'bar')

        entry.save()
        entry = Entry.objects.get(pk=entry.pk)
        self.assertEqual(entry.title, 'foo')
        self.assertEqual(entry.body, 'bar')

    def test_modification(self):
        """blogs.Entry: modification works correctly"""
        entry = Entry(title='foo', body='bar')
        entry.full_clean()
        entry.save()

        entry.title = 'foofoo'
        entry.body = 'barbar'
        entry.full_clean()
        entry.save()
        entry = Entry.objects.get(pk=entry.pk)
        self.assertEqual(entry.title, 'foofoo')
        self.assertEqual(entry.body, 'barbar')

    def test_validation(self):
        """blogs.Entry: validation works correctly"""
        from django.core.exceptions import ValidationError
        entry = Entry(title='foo', body='bar')
        entry.full_clean()
        entry.save()

        entry.title = ''
        self.assertRaises(ValidationError, entry.full_clean)

        entry.body = ''
        self.assertRaises(ValidationError, entry.full_clean)

        entry.title = '*' * 100
        self.assertRaises(ValidationError, entry.full_clean)

        entry.title = '!#$%&()'
        self.assertRaises(ValidationError, entry.full_clean)

    def test_deletion(self):
        """blogs.Entry: deletion works correctly"""
        entry = Entry(title='foo', body='bar')
        entry.full_clean()
        entry.save()

        num = Entry.objects.all().count()
        entry.delete()
        self.assertEqual(Entry.objects.all().count(), num - 1)



from __future__ import with_statement
import copy
from django.conf import settings, UserSettingsHolder
from django.utils.functional import wraps

SETTING_DELETED = object()

# Backported from Django trunk (r16377)
class override_settings(object):
    """
    Temporarily override Django settings.

    Acts as either a decorator, or a context manager.  If it's a decorator it
    takes a function and returns a wrapped function.  If it's a contextmanager
    it's used with the ``with`` statement.  In either event entering/exiting
    are called before and after, respectively, the function/block is executed.
    """
    def __init__(self, **kwargs):
        self.options = kwargs
        self.wrapped = settings._wrapped
        self._resyncdb = False

    def __enter__(self):
        self.enable()

    def __exit__(self, exc_type, exc_value, traceback):
        self.disable()

    def __call__(self, test_func):
        from django.test import TransactionTestCase
        if isinstance(test_func, type) and issubclass(test_func, TransactionTestCase):
            # When decorating a class, we need to construct a new class
            # with the same name so that the test discovery tools can
            # get a useful name.
            def _pre_setup(innerself):
                self.enable()
                test_func._pre_setup(innerself)
            def _post_teardown(innerself):
                test_func._post_teardown(innerself)
                self.disable()
            inner = type(
                test_func.__name__,
                (test_func,),
                {
                    '_pre_setup': _pre_setup,
                    '_post_teardown': _post_teardown,
                    '__module__': test_func.__module__,
                })
        else:
            @wraps(test_func)
            def inner(*args, **kwargs):
                with self:
                    return test_func(*args, **kwargs)
        return inner

    def enable(self):
        class OverrideSettingsHolder(UserSettingsHolder):
            def __getattr__(self, name):
                if name == "default_settings":
                    return self.__dict__["default_settings"]
                return getattr(self.default_settings, name)

        override = OverrideSettingsHolder(copy.copy(settings._wrapped))
        for key, new_value in self.options.iteritems():
            if new_value is SETTING_DELETED:
                try:
                    delattr(override.default_settings, key)
                except AttributeError:
                    pass
            else:
                setattr(override, key, new_value)
        settings._wrapped = override

        original_apps = frozenset(getattr(self.wrapped, 'INSTALLED_APPS'))
        override_apps = frozenset(getattr(override, 'INSTALLED_APPS'))
        if original_apps != override_apps:
            # is new app in override_apps?
            def new_app_exists():
                for app in override_apps:
                    if app not in original_apps:
                        return True
                return False
            if new_app_exists():
                # call ``syncdb`` command
                recall_syncdb()
                # clear all models meta cache if possible
                if 'django.contrib.contenttypes' in getattr(override, 'INSTALLED_APPS'):
                    clear_all_meta_caches()
                self._resyncdb = True

    def disable(self):
        settings._wrapped = self.wrapped
        if self._resyncdb:
            from django.db.models import loading
            loading.cache.loaded = False

def with_apps(*apps):
    """
    Class decorator that makes sure the passed apps are present in
    INSTALLED_APPS.
    """
    apps_set = set(settings.INSTALLED_APPS)
    apps_set.update(apps)
    return override_settings(INSTALLED_APPS=list(apps_set))

def without_apps(*apps):
    """
    Class decorator that makes sure the passed apps are not present in
    INSTALLED_APPS.
    """
    apps_list = [a for a in settings.INSTALLED_APPS if a not in apps]
    return override_settings(INSTALLED_APPS=apps_list)

def recall_syncdb():
    """call ``syncdb`` command to create tables of new app's models"""
    from django.db.models import loading
    from django.core.management import call_command
    loading.cache.loaded = False
    call_command('syncdb', interactive=False, verbosity=0, migrate=False)

def clear_meta_caches(model):
    """clear model meta caches. it is required to refresh m2m relation"""
    CACHE_NAMES = (
            '_m2m_cache', '_field_cache', '_name_map',
            '_related_objects_cache', '_related_many_to_many_cache'
        )
    for name in CACHE_NAMES:
        if hasattr(model._meta, name):
            delattr(model._meta, name)

def clear_all_meta_caches():
    """clear all models meta caches by contenttype
    
    .. Note::
        'django.contrib.contenttypes' is required to installed

    """
    from django.contrib.contenttypes.models import ContentType
    for ct in ContentType.objects.iterator():
        clear_meta_caches(ct.model_class())



# vim: set fileencoding=utf-8 :
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.http import HttpResponse
from django.test import TestCase
from django.test import RequestFactory
from mfw.tests import user_agents
from mfw.tests.override_settings import override_settings
from mfw.middleware.device import DeviceDetectionMiddleware
from mfw.middleware.session import SessionMiddleware


NOT_SUPPORT_COOKIE = {
    'HTTP_USER_AGENT': user_agents.SOFTBANK[0][5],
    'REMOTE_ADDR': user_agents.SOFTBANK[0][6],
    'HTTP_X_JPHONE_UID': 'A012345',
}
SUPPORT_COOKIE = {
    'HTTP_USER_AGENT': user_agents.SOFTBANK[1][5],
    'REMOTE_ADDR': user_agents.SOFTBANK[1][6],
    'HTTP_X_JPHONE_UID': 'A012345',
}
NOT_SUPPORT_COOKIE_NOT_RELIABLE = {
    'HTTP_USER_AGENT': user_agents.SOFTBANK[0][5],
    'REMOTE_ADDR': '127.0.0.1',
    'HTTP_X_JPHONE_UID': 'A012345',
}
NOT_SUPPORT_COOKIE_DOCOMO = {
    'HTTP_USER_AGENT': user_agents.DOCOMO[0][5],
    'REMOTE_ADDR': user_agents.DOCOMO[0][6],
    'HTTP_X_DCMGUID': 'A012345',
}


def create_session_key(meta):
    from mfw.device import detect
    from mfw.middleware.session import create_session_key as fn
    device = detect(meta)
    return fn(device.carrier, device.uid)


class MFWSessionMiddlewareTestCase(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_session_with_support_cookie(self):
        request = self.factory.get('/', **SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()
        session_key = create_session_key(SUPPORT_COOKIE)

        middleware1.process_request(request)
        middleware2.process_request(request)

        # Add new session key
        request.session['hello'] = 'world'

        response = middleware2.process_response(request, response)

        # request bundle session should not use UID for session key
        self.assertNotEqual(request.session.session_key, session_key)
        # request bundle COOKIES should not be overridden
        self.assertFalse(getattr(request.COOKIES, '_mfw_overridden', False))
        # response set_cookie method should not be overridden
        self.assertFalse(getattr(response.set_cookie, '_mfw_overridden', False))

        # start new request
        request = self.factory.get('/', **NOT_SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1.process_request(request)
        middleware2.process_request(request)
        response = middleware2.process_response(request, response)

        # session should be saved correctly
        # but django's SessionMiddleware use cookies to store generated
        # session_key and RequestFactory seems to not handle cookie
        # thus actually session is not saved with in this test.
        #self.assertEqual(request.session.get('hello', None), 'world')

    def test_session_with_not_support_cookie(self):
        request = self.factory.get('/', **NOT_SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()
        session_key = create_session_key(NOT_SUPPORT_COOKIE)

        middleware1.process_request(request)
        middleware2.process_request(request)

        # Add new session key
        request.session['hello'] = 'world'

        response = middleware2.process_response(request, response)

        # request bundle session should use UID for session key
        self.assertEqual(request.session.session_key, session_key)
        # request bundle COOKIES should be overridden
        self.assertTrue(getattr(request.COOKIES, '_mfw_overridden', False))
        # response set_cookie method should be overridden
        self.assertTrue(getattr(response.set_cookie, '_mfw_overridden', False))

        # start new request
        request = self.factory.get('/', **NOT_SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1.process_request(request)
        middleware2.process_request(request)
        response = middleware2.process_response(request, response)

        # session should be saved correctly
        self.assertEqual(request.session.get('hello', None), 'world')

    def test_session_with_not_support_cookie_not_reliable(self):
        with override_settings(MFW_SESSION_TRUST_NON_RELIABLE_DEVICE=False):
            request = self.factory.get('/', **NOT_SUPPORT_COOKIE_NOT_RELIABLE)
            response = HttpResponse('Session test')
            middleware1 = DeviceDetectionMiddleware()
            middleware2 = SessionMiddleware()
            session_key = create_session_key(NOT_SUPPORT_COOKIE_NOT_RELIABLE)

            middleware1.process_request(request)
            middleware2.process_request(request)

            # Add new session key
            request.session['hello'] = 'world'

            response = middleware2.process_response(request, response)

            # request bundle session should not use UID for session key
            # because the device is not reliable
            self.assertNotEqual(request.session.session_key, session_key)
            # request bundle COOKIES should not be overridden
            # because the device is not reliable
            self.assertFalse(getattr(request.COOKIES, '_mfw_overridden', False))
            # response set_cookie method should not be overridden
            # because the device is not reliable
            self.assertFalse(getattr(response.set_cookie, '_mfw_overridden', False))

        with override_settings(MFW_SESSION_TRUST_NON_RELIABLE_DEVICE=True):
            request = self.factory.get('/', **NOT_SUPPORT_COOKIE_NOT_RELIABLE)
            response = HttpResponse('Session test')
            middleware1 = DeviceDetectionMiddleware()
            middleware2 = SessionMiddleware()
            session_key = create_session_key(NOT_SUPPORT_COOKIE_NOT_RELIABLE)

            middleware1.process_request(request)
            middleware2.process_request(request)

            # Add new session key
            request.session['hello'] = 'world'

            response = middleware2.process_response(request, response)

            # request bundle session should use UID for session key
            # because the device is not reliable but
            # MFW_SESSION_TRUST_NON_RELIABLE_DEVICE set True
            self.assertEqual(request.session.session_key, session_key)
            # request bundle COOKIES should not be overridden
            # because the device is not reliable but
            # MFW_SESSION_TRUST_NON_RELIABLE_DEVICE set True
            self.assertTrue(getattr(request.COOKIES, '_mfw_overridden', False))
            # response set_cookie method should not be overridden
            # because the device is not reliable but
            # MFW_SESSION_TRUST_NON_RELIABLE_DEVICE set True
            self.assertTrue(getattr(response.set_cookie, '_mfw_overridden', False))

    def test_session_with_not_support_cookie_docomo(self):
        d = dict(NOT_SUPPORT_COOKIE_DOCOMO)
        del d['HTTP_X_DCMGUID']
        request = self.factory.get('/', **d)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()

        middleware1.process_request(request)
        response = middleware2.process_request(request)

        # without '?guid=pn', DoCoMo server does not return UID thus
        # the middleware redirect when no ?guid=on is found
        self.assertNotEqual(response, None)
        response.client = self.client   # monkey patch
        self.assertRedirects(response, '/?guid=on')

        request = self.factory.get('/?guid=on', **NOT_SUPPORT_COOKIE_DOCOMO)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()

        middleware1.process_request(request)
        response = middleware2.process_request(request)

        # with guid=on, the middleware should not redirect
        self.assertEqual(response, None)

    def test_csrf_protection_pass_with_not_support_cookie(self):
        from django.middleware.csrf import get_token
        from mfw.middleware.csrf import CsrfViewMiddleware

        # request with GET
        request = self.factory.get('/', **NOT_SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()
        middleware3 = CsrfViewMiddleware()

        middleware1.process_request(request)
        middleware2.process_request(request)
        middleware3.process_view(request, None, (), {})

        # Simulate {% csrf_token %} templatetag. It is required to
        # set request.META['CSRF_COOKIE_USED'] = True
        csrf_token = str(get_token(request))
        self.assertEqual(request.META.get('CSRF_COOKIE', None), csrf_token)
        self.assertEqual(request.META.get('CSRF_COOKIE_USED', False), True)

        # bottom up for response phase
        response = middleware3.process_response(request, response)
        response = middleware2.process_response(request, response)
        
        # new request with POST
        data = {'csrfmiddlewaretoken': csrf_token}
        request = self.factory.post('/', data, **NOT_SUPPORT_COOKIE)

        middleware1.process_request(request)
        middleware2.process_request(request)
        response = middleware3.process_view(request, None, (), {})

        # if csrf protection worked, response will be HttpResponseForbidden
        self.assertEqual(request.META.get('CSRF_COOKIE', None), csrf_token)
        self.assertEqual(request.META.get('CSRF_COOKIE_USED', False), False)
        self.assertEqual(response, None)

    def test_csrf_protection_fail_with_not_support_cookie(self):
        from django.http import HttpResponseForbidden
        from django.middleware.csrf import get_token
        from mfw.middleware.csrf import CsrfViewMiddleware

        # request with GET
        request = self.factory.get('/', **NOT_SUPPORT_COOKIE)
        response = HttpResponse('Session test')
        middleware1 = DeviceDetectionMiddleware()
        middleware2 = SessionMiddleware()
        middleware3 = CsrfViewMiddleware()

        middleware1.process_request(request)
        middleware2.process_request(request)
        middleware3.process_view(request, None, (), {})

        # Simulate {% csrf_token %} templatetag. It is required to
        # set request.META['CSRF_COOKIE_USED'] = True
        csrf_token = str(get_token(request))
        self.assertEqual(request.META.get('CSRF_COOKIE', None), csrf_token)
        self.assertEqual(request.META.get('CSRF_COOKIE_USED', False), True)

        # bottom up for response phase
        response = middleware3.process_response(request, response)
        response = middleware2.process_response(request, response)

        # new request with POST
        data = {'csrfmiddlewaretoken': 'wrong-csrf-token'}
        request = self.factory.post('/', data, **NOT_SUPPORT_COOKIE)

        middleware1.process_request(request)
        middleware2.process_request(request)
        response = middleware3.process_view(request, None, (), {})

        # if csrf protection worked, response will be HttpResponseForbidden
        self.assertEqual(request.META.get('CSRF_COOKIE', None), csrf_token)
        self.assertEqual(request.META.get('CSRF_COOKIE_USED', False), False)
        self.assertNotEqual(response, None)
        self.assertEqual(response.status_code, 403)
        self.assertTrue(isinstance(response, HttpResponseForbidden))



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
"""
Mini blog forms


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django import forms

from models import Entry

class EntryForm(forms.ModelForm):
    class Meta:
        model = Entry
        fields = ('title', 'body')



# vim: set fileencoding=utf-8 :
from setuptools import setup, find_packages

version = "0.3.1"

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()

setup(
    name="django-mfw",
    version=version,
    description = "Django framework for mobilephone/smartphone site",
    long_description=read('README.rst'),
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = "django mobile smartphone emoji device detection dynamic template",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/django-mfw",
    download_url = r"https://github.com/lambdalisue/django-mfw/tarball/master",
    license = 'MIT',
    packages = find_packages(),
    include_package_data = True,
    zip_safe = True,
    install_requires=[
        'django>=1.3',
        'distribute',
        'setuptools-git',
        'IPy',
        'BeautifulSoup',
        'e4u',
    ],
    test_suite='runtests.runtests',
    tests_require=[
        'PyYAML',
    ],
)



# vim: set fileencoding=utf-8 :
"""
Flavour template loader


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import os
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.loader import BaseLoader
from django.template.loader import find_template_loader

from mfw.middleware.flavour import get_flavour
    
class Loader(BaseLoader):
    """Flavour template loader"""
    is_usable = True

    def __init__(self, *args, **kwargs):
        loaders = []
        for loader_name in settings.TEMPLATE_LOADERS:
            if loader_name == 'mfw.template.loaders.flavour.Loader':
                continue
            loader = find_template_loader(loader_name)
            if loader is not None:
                loaders.append(loader)
        self.template_source_loaders = tuple(loaders)
        super(Loader, self).__init__(*args, **kwargs)

    def prepare_template_name(self, template_name, flavour):
        """prepare template_name with flavour and actual template name"""
        return os.path.join(flavour, template_name)

    def get_flavour(self):
        return get_flavour()

    def load_template(self, template_name, template_dirs=None):
        tried = []
        flavour = self.get_flavour()
        while(True):
            _template_name = self.prepare_template_name(template_name, flavour)
            for loader in self.template_source_loaders:
                try:
                    return loader.load_template(_template_name, template_dirs)
                except TemplateDoesNotExist:
                    tried.append(_template_name)
            # slice up flavour (include flavor='')
            if flavour:
                flavour = os.path.dirname(flavour)
            else:
                break
        if tried:
            error_msg = "Tried %s" % tried
        else:
            error_msg = "Your TEMPLATE_DIRS settings is empty. Change it to point to at least one template directory."
        raise TemplateDoesNotExist(error_msg)

    def load_template_source(self, template_name, template_dirs=None):
        tried = []
        flavour = self.get_flavour()
        while(True):
            _template_name = self.prepare_template_name(template_name, flavour)
            for loader in self.template_source_loaders:
                try:
                    return loader.load_template_source(_template_name, template_dirs)
                except TemplateDoesNotExist:
                    tried.append(_template_name)
            # slice up flavour (include flavor='')
            if flavour:
                flavour = os.path.dirname(flavour)
            else:
                break
        if tried:
            error_msg = "Tried %s" % tried
        else:
            error_msg = "Your TEMPLATE_DIRS settings is empty. Change it to point to at least one template directory."
        raise TemplateDoesNotExist(error_msg)
    load_template_source.is_usable = True



import os
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

def setconf(name, default_value):
    value = getattr(settings, name, default_value)
    setattr(settings, name, value)

setconf('MFW_EMOJI_TRANSLATION_FILENAME', os.path.join(os.path.dirname(__file__), 'data', 'emoji4unicode.xml'))
setconf('MFW_EMOJI_DEFAULT_CARRIER', 'kddi_img')
setconf('MFW_EMOJI_DEFAULT_ENCODING', 'utf-8')

# Load e4u
import e4u
e4u.load(filename=settings.MFW_EMOJI_TRANSLATION_FILENAME)

# Validation
if 'mfw.contrib.emoji.middleware.DeviceEmojiTranslationMiddleware' in settings.MIDDLEWARE_CLASSES:
    found = False
    for middleware in settings.MIDDLEWARE_CLASSES:
        if middleware == 'mfw.middleware.encoding.DeviceEncodingMiddleware':
            found = True
        elif middleware == 'mfw.contrib.emoji.middleware.DeviceEmojiTranslationMiddleware' and not found:
            raise ImproperlyConfigured("``mfw.middleware.encoding.DeviceEncodingMiddleware`` must be in ``MIDDLEWARE_CLASSES`` before the ``mfw.contrib.emoji.middleware.DeviceEmojiTranslationMiddleware``")



from django.conf import settings

def setconf(name, default_value):
    value = getattr(settings, name, default_value)
    setattr(settings, name, value)

# A dictionary of dotted python path of CIDR scrapper classes.
# The key of the dictionary is carrier name used to determine
# which scrapper should be used to scrape CIDR
setconf('MFW_CIDR_SCRAPER_CLASSES', {
        'docomo': 'mfw.cidr.scrapers.DoCoMoCIDRScraper',
        'kddi': 'mfw.cidr.scrapers.KDDICIDRScraper',
        'softbank': 'mfw.cidr.scrapers.SoftbankCIDRScraper',
    })

# Expiration days of each CIDR. Each CIDR will be updated if
# the CIDR is expired.
setconf('MFW_CIDR_EXPIRATION_DAYS', 30)

# A list of dotted python path of Device classes.
# Add your own Device class when builtin devices are not enough
# to use (but I suggest you to contribute :-) Contribution is always welcome.)
setconf('MFW_DEVICE_CLASSES', (
        'mfw.device.smartphone.Smartphone',
        'mfw.device.browser.Browser',
        'mfw.device.mobilephone.DoCoMo',
        'mfw.device.mobilephone.KDDI',
        'mfw.device.mobilephone.Softbank',
    ))

# Checking reliable makes the response slow sometime. You can turned off
# the feature with the setting below.
setconf('MFW_CHECK_DEVICE_RELIABLE', True)

# mfw SessionMiddleware does not trust non reliable device and does not use
# UID based SessionMiddleware in that case because of security.
# However if you desire to turned off this feature and accept all non cookie
# supported device, use the setting below but STRONGLY NOT RECOMMENDED.
setconf('MFW_SESSION_TRUST_NON_RELIABLE_DEVICE', False)

# Used to create flavour from device. The function receive ``device`` instance
# and must return list/tuple
setconf('MFW_DEVICE_FLAVOUR_COLUMNS', lambda device: (device.kind.lower(), device.name.lower(), device.model.lower(), device.version.lower()))

# Used to create flavour from device. Each list item has ``condition`` and
# ``columns`` function and when ``condition`` function return ``True`` then
# ``columns`` function is used to overlap existing columns. Both function
# receive ``device`` and ``previous`` and must return list/tuple.
setconf('MFW_DEVICE_FLAVOUR_OVERLAP_RULES', ())



# vim: set fileencoding=utf-8 :
"""
Device loader


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from mfw.device.base import Device
from mfw.device.base import DummyDevice

try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module


def get_device_class(path=None):
    """
    Return an class of a device, given the dotted
    Python import path (as a string) to the device class.

    If the device cannot be located (e.g., because no such module
    exists, or because the module does not contain a class of the
    appropriate name), ``django.core.exceptions.ImproperlyConfigured``
    is raised.
    
    """
    i = path.rfind('.')
    module, attr = path[:i], path[i+1:]
    try:
        mod = import_module(module)
    except ImportError, e:
        raise ImproperlyConfigured('Error loading mfw device %s: "%s"' % (module, e))
    try:
        cls = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a mfw device named "%s"' % (module, attr))
    if cls and not issubclass(cls, Device):
        raise ImproperlyConfigured('Device class "%s" must be a subclass of ``mfw.device.base.Device``' % path)
    return cls


class DeviceDetector(object):
    def __init__(self):
        self._device_classes = []
        for device_class in settings.MFW_DEVICE_CLASSES:
            self._device_classes.append(get_device_class(device_class))

    def detect(self, meta):
        for device_class in self._device_classes:
            device = device_class.detect(meta)
            if device:
                return device
        return DummyDevice()
